--  BSP.SIO -- Serial Input Output (Low level and polling driven)
--  Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
--  Written by Stephane Carrez (stcarrez@nerim.fr)
--
--  This file is free software; you can redistribute it and/or modify it
--  under the terms of the GNU General Public License as published by the
--  Free Software Foundation; either version 2, or (at your option) any
--  later version.
--
--  In addition to the permissions in the GNU General Public License, the
--  Free Software Foundation gives you unlimited permission to link the
--  compiled version of this file with other programs, and to distribute
--  those programs without any restriction coming from the use of this
--  file.  (The General Public License restrictions do apply in other
--  respects; for example, they cover modification of the file, and
--  distribution when not linked into another program.)
--
--  This file is distributed in the hope that it will be useful, but
--  WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; see the file COPYING.  If not, write to
--  the Free Software Foundation, 59 Temple Place - Suite 330,
--  Boston, MA 02111-1307, USA.
pragma No_Run_Time;
package BSP.SIO is

   procedure Initialize;
   --  Initializes the SCI.
   --  This procedure configures the SCI to send at \c M6811_DEF_BAUD baud
   --  rate.  It must be called before any other serial operation unless
   --  the program is started from another one (such as a monitor or
   --  in bootstrap mode).
   pragma Inline_Always (Initialize);

   function Receive_Pending return Boolean;
   --  Return true if there is a character to read from the SCI.
   pragma Inline_Always (Receive_Pending);

   procedure Flush;
   --  Flush the SCI send queue.
   --  This procedure waits for the transmission of the current character.
   --  While waiting, the COP is reset.

   procedure Send (C : in Character);
   --  Send the character on the serial line.
   --  Before sending, it calls \c Flush to make sure the
   --  transmitter is ready.  Once the function returns, the
   --  character is in the SCI queue and it may not be sent completely
   --  over the serial line.

   procedure Send (S : in String);
   --  Send the string on the serial line.

   function Receive return Character;
   --  Wait and return the character received on the SCI.

end BSP.SIO;
