/* malloc.h -- Malloc Internal Data structures.
   Copyright 2003 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@nerim.fr)

This file is part of GEL.

GEL is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GEL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GEL; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#ifndef _LIB_MALLOC_H
#define _LIB_MALLOC_H

#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

/*

_malloc_bottom
  |
  |  +--------------+   +-----------------+   +----------+
  |  |              |   |                 |   |          |
  v  |              v   |                 v   |          v
  +--------+--------+-------+-------------+------+-------+------+-----------+
  | next   | Block  | next  | next_free   | next | Block | next | next_free |
  +--------+--------+-------+-------------+------+-------+------+-----------+
  ^        ^        ^           |                        ^
  |        |        |           |                        |
  |    User pointer |           +------------------------+
  Slot              |
  <---------------->|<-------------------> <-------------> <---------------->
      Allocated     |         Free           Allocated            Free
              _malloc_first
*/

struct slot 
{
  struct slot* next;
  struct slot* next_free;
};


extern struct slot* _malloc_bottom;
extern struct slot* _malloc_first;
extern void* _malloc_top;

#endif
