/*-
 * Copyright (c) 1990, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * Chris Torek.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)local.h	8.3 (Berkeley) 7/3/94
 * $FreeBSD: src/lib/libc/stdio/local.h,v 1.21 2002/10/25 07:01:56 tjr Exp $
 */

#include <sys/types.h>	/* for off_t */
#ifdef HAVE_THREADS
#include <pthread.h>
#endif
#include <string.h>
#ifdef HAVE_WCHAR
#include <wchar.h>
#endif

#include <unistd.h>

/*
 * Information local to this implementation of stdio,
 * in particular, macros and private variables.
 */

extern int	_sread(FILE *, char *, int);
extern int	_swrite(FILE *, char const *, int);
extern fpos_t	_sseek(FILE *, fpos_t, int);
extern int	_ftello(FILE *, fpos_t *);
extern int	_fseeko(FILE *, off_t, int, int);
extern int	__fflush(FILE *fp);
#ifdef HAVE_WCHAR
extern wint_t	__fgetwc(FILE *);
extern wint_t	__fputwc(wchar_t, FILE *);
#endif
extern int	__sflush(FILE *);
extern FILE	*__sfp(void);
extern int	__srefill(FILE *);
extern int	__sread(void *, char *, int);
extern int	__swrite(void *, char const *, int);
extern fpos_t	__sseek(void *, fpos_t, int);
extern int	__sclose(void *);
extern void	__sinit(void);
extern void	_cleanup(void);
extern void	(*__cleanup)(void);
extern void	__smakebuf(FILE *);
extern int	__swhatbuf(FILE *, size_t *, int *);
extern int	_fwalk(int (*)(FILE *));
extern int	__svfscanf(FILE *, const char *, __va_list);
extern int	__swsetup(FILE *);
extern int	__sflags(const char *, int *);
extern int	__ungetc(int, FILE *);
#ifdef HAVE_WCHAR
extern wint_t	__ungetwc(wint_t, FILE *);
#endif
extern int	__vfprintf(FILE *, const char *, __va_list);
extern int	__vfscanf(FILE *, const char *, __va_list);
#ifdef HAVE_WCHAR
extern int	__vfwprintf(FILE *, const wchar_t *, __va_list);
extern int	__vfwscanf(FILE * __restrict, const wchar_t * __restrict,
		    __va_list);
#endif

extern int	__sdidinit;

struct __File_ops {
  int (* flushbuf) (char c, FILE* fp);
  int (* fflush) (FILE* fp);
};

#ifdef HAVE_THREADS
/* hold a buncha junk that would grow the ABI */
struct __sFILEX {
	unsigned char	*_up;	/* saved _p when _p is doing ungetc data */
#ifdef HAVE_THREADS
	pthread_mutex_t	fl_mutex;	/* used for MT-safety */
	pthread_t	fl_owner;	/* current owner */
	int		fl_count;	/* recursive lock count */
#endif
	int		orientation;	/* orientation for fwide() */
#ifdef notdef
	/*
	 * XXX These are not used yet -- they will be used to store the
	 * multibyte conversion state for writing and reading when
	 * stateful encodings are supported by the locale framework.
	 */
	mbstate_t	wstate;		/* write conversion state */
	mbstate_t	rstate;		/* read conversion state */
#endif
};
#endif
/*
 * Return true iff the given FILE cannot be written now.
 */
#define	cantwrite(fp) \
 	((((fp)->_flags & __SWR) == 0 || \
 	    ((fp)->_bf._base == NULL && ((fp)->_flags & __SSTR) == 0)) && \
	 __swsetup(fp))

/*
 * Test whether the given stdio file has an active ungetc buffer;
 * release such a buffer, without restoring ordinary unread data.
 */
#define	HASUB(fp) ((fp)->_ub._base != NULL)
#define	FREEUB(fp) { \
	/*SCz: if ((fp)->_ub._base != (fp)->_ubuf)*/ \
		free((char *)(fp)->_ub._base); \
	(fp)->_ub._base = NULL; \
}

/*
 * test for an fgetln() buffer.
 */
#define	HASLB(fp) ((fp)->_lb._base != NULL)
#define	FREELB(fp) { \
	free((char *)(fp)->_lb._base); \
	(fp)->_lb._base = NULL; \
}

#define	INITEXTRA(fp) { \
	(fp)->_extra->_up = NULL; \
	(fp)->_extra->orientation = 0; \
	/* memset(&(fp)->_extra->wstate, 0, sizeof(mbstate_t)); */ \
	/* memset(&(fp)->_extra->rstate, 0, sizeof(mbstate_t)); */ \
}

#ifdef SCz
/*
 * Set the orientation for a stream. If o > 0, the stream has wide-
 * orientation. If o < 0, the stream has byte-orientation.
 */
#define	ORIENT(fp, o)	do {				\
	if ((fp)->_extra->orientation == 0)		\
		(fp)->_extra->orientation = (o);	\
} while (0)
#else
#define ORIENT(fp,o)
#endif
