/* button-test.c -- Test of buttons
   Copyright 2000 Free Software Foundation, Inc.
   Written by Stephane Carrez (stcarrez@worldnet.fr)

This file is part of EBCS.

EBCS is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

EBCS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with EBCS; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <sys/param.h>
#include <sys/locks.h>
#include <gel/timer.h>
#include <sys/sio.h>
#include <gel/buttons.h>
#include <ebcs/remote.h>
#include <ebcs/board.h>
#include <benchs.h>
#include <stdio.h>
#include <test.h>
#include <lcd.h>
#include <ebcs/triacs.h>

#define EVENT_QUEUE_SIZE 16

static event event_queue[EVENT_QUEUE_SIZE];

static unsigned short prev_report;

event_callout test_button_callout;

long value;
unsigned short values[3];
unsigned char col = 2;
unsigned short *pvalue = &values[0];
unsigned char line = 1;

void
button_event_handler (event_callout *callout, event *ev)
{
  unsigned short mask = button_get_pressed ();
  unsigned short delta;
  int refresh_cursor = 0;
  int refresh_text = 0;
  
  delta = mask ^ prev_report;
  prev_report = mask;

  if (delta & 1 & mask)
    {
      if (col == 2)
        {
          col = 5;
          pvalue = &values[1];
        }
      else if (col == 5)
        {
          col = 8;
          pvalue = &values[2];
        }
      else
        {
          col = 2;
          pvalue = &values[0];
        }
      refresh_cursor = 1;
    }
  if (delta & 2 & mask)
    {
      pvalue[0]++;
      refresh_text = 1;
    }

  if (delta & 4 & mask)
    {
      if (triac_get (TRIAC_FAN) == TRIAC_ON)
        triac_set (TRIAC_FAN, TRIAC_OFF);
      else
        triac_set (TRIAC_FAN, TRIAC_ON);
    }
  
  if (refresh_text || refresh_cursor)
    {
      char buf[32];
      
      button_disable ();
      if (refresh_text)
        {
          sprintf (buf, " %c%c:%c%c:%c%c",
                   ((values[0] / 10) % 10) + '0',
                   ((values[0] % 10) + '0'),
                   ((values[1] / 10) % 10) + '0',
                   ((values[1] % 10) + '0'),
                   ((values[2] / 10) % 10) + '0',
                   ((values[2] % 10) + '0'));
          lcd_print_at (0, 1, buf);
        }
      lcd_goto (col, 1);
      button_enable ();
    }
}

unsigned char stop = 0;

int
main()
{
  serial_init ();
  remote_initialize ();

  timer_initialize ();
  event_initialize (event_queue, EVENT_QUEUE_SIZE);
  button_initialize ();
  lcd_reset ();
  lcd_print_at (0, 0, "Hello World!   ");
  button_enable ();

  test_depend ("serial:timer:event");
  test_start ("lcd-button");

  event_register (&button_change_event, &test_button_callout,
                  button_event_handler, 0);

  event_loop (1000 * 31 * 65535, &stop);
  test_end ("lcd-button");
  return 0;
}
